package com.yinet.common.config;

import java.io.FileNotFoundException;
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.yinet.common.utils.StringUtils;
import com.yinet.common.utils.YamlUtil;

/**
 * 全局配置类
 *
 * @author yinet
 */
public class Global {
    private static final Logger log = LoggerFactory.getLogger(Global.class);

    private static String NAME = "application.yml";

    /**
     * 当前对象实例
     */
    private static Global global;

    /**
     * 保存全局属性值
     */
    private static Map<String, String> map = new HashMap<String, String>();

    private Global() {
    }

    /**
     * 静态工厂方法
     */
    public static synchronized Global getInstance() {
        if (global == null) {
            global = new Global();
        }
        return global;
    }

    /**
     * 获取配置
     */
    public static String getConfig(String key) {
        String value = map.get(key);
        if (value == null) {
            Map<?, ?> yamlMap = null;
            try {
                yamlMap = YamlUtil.loadYaml(NAME);
                value = String.valueOf(YamlUtil.getProperty(yamlMap, key));
                map.put(key, value != null ? value : StringUtils.EMPTY);
            } catch (FileNotFoundException e) {
                log.error("获取全局配置异常 {}", key);
            }
        }
        return value;
    }

    /**
     * 获取项目名称
     */
    public static String getName() {
        return StringUtils.nvl(getConfig("yinet.name"), "yinet");
    }

    /**
     * 获取其他信息
     */
    public static String getOther(String item) {
        return StringUtils.nvl(getConfig("yinet." + item), null);
    }

    /**
     * 获取项目版本
     */
    public static String getVersion() {
        return StringUtils.nvl(getConfig("yinet.version"), "4.0.0");
    }

    /**
     * 获取版权年份
     */
    public static String getCopyrightYear() {
        return StringUtils.nvl(getConfig("yinet.copyrightYear"), "2019");
    }

    /**
     * 实例演示开关
     */
    public static String isDemoEnabled() {
        return StringUtils.nvl(getConfig("yinet.demoEnabled"), "true");
    }

    /**
     * 获取ip地址开关
     */
    public static Boolean isAddressEnabled() {
        return Boolean.valueOf(getConfig("yinet.addressEnabled"));
    }

    /**
     * 获取文件上传路径
     */
    public static String getProfile() {
        return getConfig("yinet.profile");
    }

    /**
     * 获取头像上传路径
     */
    public static String getAvatarPath() {
        return getProfile() + "/avatar";
    }

    /**
     * 获取下载路径
     */
    public static String getDownloadPath() {
        return getProfile() + "/download";
    }

    /**
     * 获取上传路径
     */
    public static String getUploadPath() {
        return getProfile() + "/upload";
    }

    /**
     * 获取公示公告预览图路径
     */
    public static String getNoticeImgPath() {
        return getProfile() + "/upload/chumo/notice/img";
    }

    /**
     * 获取办事指南文档路径
     */
    public static String getGuidanceDocPath() {
        return getProfile() + "/upload/chumo/guidance/doc";
    }

    /**
     * 获取办事指南科室图标路径
     */
    public static String getGuidanceIconPath() {
        return getProfile() + "/upload/chumo/guidance/icon";
    }

    /**
     * 获取日志路径
     */
    public static String getLogHbzwfwFilePath() {
        return getProfile() + "/logs/hbzwfw/";
    }

    /**
     * 获取微信公众号APPID
     */
    public static String getWxAppid() {
        return StringUtils.nvl(getConfig("wxconfig.appid"), "");
    }

    /**
     * 获取微信公众号AppSecret
     */
    public static String getWxAppSecret() {
        return StringUtils.nvl(getConfig("wxconfig.appsecret"), "");
    }

    public static String getKKFileView() {
        return StringUtils.nvl(getConfig("kkfileview.url"), "http://127.0.0.1:9001/");
    }
}
