package com.yinet.review.service.impl;

import com.yinet.common.core.text.Convert;
import com.yinet.common.exception.BusinessException;
import com.yinet.common.utils.StringUtils;
import com.yinet.review.domain.FormReview;
import com.yinet.review.mapper.FormReviewMapper;
import com.yinet.review.service.IFormReviewService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 评审项目Service业务层处理
 * 
 * @author yinet
 * @date 2020-04-28
 */
@Service
public class FormReviewServiceImpl implements IFormReviewService {
    @Autowired
    private FormReviewMapper formReviewMapper;

    public static final Logger log = LoggerFactory.getLogger(FormReviewServiceImpl.class);
    /**
     * 查询评审项目
     *
     * @param id 评审项目ID
     * @return 评审项目
     */
    @Override
    public FormReview selectFormReviewById(Long id) {
        return formReviewMapper.selectFormReviewById(id);
    }

    /**
     * 查询评审项目列表
     *
     * @param formReview 评审项目
     * @return 评审项目
     */
    @Override
    public List<FormReview> selectFormReviewList(FormReview formReview) {
        return formReviewMapper.selectFormReviewList(formReview);
    }

    /**
     * 新增评审项目
     *
     * @param formReview 评审项目
     * @return 结果
     */
    @Override
    public int insertFormReview(FormReview formReview) {
        return formReviewMapper.insertFormReview(formReview);
    }

    /**
     * 修改评审项目
     *
     * @param formReview 评审项目
     * @return 结果
     */
    @Override
    public int updateFormReview(FormReview formReview) {
        return formReviewMapper.updateFormReview(formReview);
    }

    /**
     * 删除评审项目对象
     *
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    @Override
    public int deleteFormReviewByIds(String ids) {
        return formReviewMapper.deleteFormReviewByIds(Convert.toStrArray(ids));
    }

    /**
     * 删除评审项目信息
     *
     * @param id 评审项目ID
     * @return 结果
     */
    public int deleteFormReviewById(Long id) {
        return formReviewMapper.deleteFormReviewById(id);
    }

    /**
     * @param formReviewsList 用户数据列表
     * @param updateSupport   是否更新支持，如果已存在，则进行更新数据
     * @return
     */
    @Override
    public String importFormReviewsList(List<FormReview> formReviewsList, boolean updateSupport) {
        if (StringUtils.isNull(formReviewsList) || formReviewsList.size() == 0) {
            throw new BusinessException("导入用户数据不能为空！");
        }
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();

        for (FormReview formReview : formReviewsList) {
            try {

                this.insertFormReview(formReview);

                successNum++;
            } catch (Exception e) {
                failureNum++;
                String msg = "<br/>" + failureNum + "、账号 " + formReview.getProjectname() + " 导入失败：";
                failureMsg.append(msg + e.getMessage());
               log.error(msg, e);
            }
        }

        if (failureNum > 0) {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new BusinessException(failureMsg.toString());
        } else {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
        }
        return successMsg.toString();
       /* int i=0;
        for(GtgsywOld old:olds){
                this.insertGtgsywOld(old);
            i++;
        }
        return  i;*/
    }
}

//        for (FormReview formReview : formReviewsList) {
//            try {
//                //验证数据是否存在
//              //  FormReview fr = userMapper.selectUserByLoginName(user.getLoginName());
//                if (StringUtils.isNull(fr)) {
//                    user.setPassword(Md5Utils.hash(user.getLoginName() + password));
//                    user.setCreateBy(operName);
//                    this.insertUser(user);
//                    successNum++;
//                    successMsg.append("<br/>" + successNum + "、账号 " + user.getLoginName() + " 导入成功");
//                } else if (isUpdateSupport) {
//                    user.setUpdateBy(operName);
//                    this.updateUser(user);
//                    successNum++;
//                    successMsg.append("<br/>" + successNum + "、账号 " + user.getLoginName() + " 更新成功");
//                } else {
//                    failureNum++;
//                    failureMsg.append("<br/>" + failureNum + "、账号 " + user.getLoginName() + " 已存在");
//                }
//            } catch (Exception e) {
//                failureNum++;
//                String msg = "<br/>" + failureNum + "、账号 " + user.getLoginName() + " 导入失败：";
//                failureMsg.append(msg + e.getMessage());
//                log.error(msg, e);
//            }
//        }
//        if (failureNum > 0) {
//            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
//            throw new BusinessException(failureMsg.toString());
//        } else {
//            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
//        }
//        return successMsg.toString();
//        return null;
//    }
//}
