package com.yinet.system.service.impl;

import java.util.List;

import com.yinet.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.yinet.system.mapper.GuidanceFileMapper;
import com.yinet.system.domain.GuidanceFile;
import com.yinet.system.service.IGuidanceFileService;
import com.yinet.common.core.text.Convert;

/**
 * 办事指南文件Service业务层处理
 *
 * @author yinet
 * @date 2019-11-11
 */
@Service
public class GuidanceFileServiceImpl implements IGuidanceFileService {
    @Autowired
    private GuidanceFileMapper guidanceFileMapper;

    /**
     * 查询办事指南文件
     *
     * @param fileId 办事指南文件ID
     * @return 办事指南文件
     */
    @Override
    public GuidanceFile selectGuidanceFileById(String fileId) {
        return guidanceFileMapper.selectGuidanceFileById(fileId);
    }

    @Override
    public List<GuidanceFile> selectGuidanceFileByGuidanceId(String guidanceId) {
        return guidanceFileMapper.selectGuidanceFileByGuidanceId(guidanceId);
    }

    /**
     * 查询办事指南文件列表
     *
     * @param guidanceFile 办事指南文件
     * @return 办事指南文件
     */
    @Override
    public List<GuidanceFile> selectGuidanceFileList(GuidanceFile guidanceFile) {
        return guidanceFileMapper.selectGuidanceFileList(guidanceFile);
    }

    /**
     * 新增办事指南文件
     *
     * @param guidanceFile 办事指南文件
     * @return 结果
     */
    @Override
    public int insertGuidanceFile(GuidanceFile guidanceFile) {
        guidanceFile.setCreateTime(DateUtils.getNowDate());
        return guidanceFileMapper.insertGuidanceFile(guidanceFile);
    }

    /**
     * 修改办事指南文件
     *
     * @param guidanceFile 办事指南文件
     * @return 结果
     */
    @Override
    public int updateGuidanceFile(GuidanceFile guidanceFile) {
        guidanceFile.setUpdateTime(DateUtils.getNowDate());
        return guidanceFileMapper.updateGuidanceFile(guidanceFile);
    }

    /**
     * 删除办事指南文件对象
     *
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    @Override
    public int deleteGuidanceFileByIds(String ids) {
        return guidanceFileMapper.deleteGuidanceFileByIds(Convert.toStrArray(ids));
    }

    /**
     * 删除办事指南文件信息
     *
     * @param fileId 办事指南文件ID
     * @return 结果
     */
    @Override
    public int deleteGuidanceFileById(String fileId) {
        return guidanceFileMapper.deleteGuidanceFileById(fileId);
    }
}
